package eu.mhsl.craftattack.spawn.appliances.whitelist;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.api.client.repositories.WhitelistRepository;
import eu.mhsl.craftattack.spawn.api.server.HttpServer;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.outlawed.Outlawed;
import eu.mhsl.craftattack.spawn.util.api.HttpStatus;
import eu.mhsl.craftattack.spawn.util.server.Floodgate;
import eu.mhsl.craftattack.spawn.util.text.DisconnectInfo;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.time.Instant;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;

public class Whitelist extends Appliance {
    public Whitelist() {
        super("whitelist");
    }

    public void integrityCheck(Player player) throws DisconnectInfo.Throwable {
        try {
            Main.instance().getLogger().info(String.format("Running integrityCheck for %s", player.getName()));
            boolean overrideCheck = localConfig().getBoolean("overrideIntegrityCheck", false);
            WhitelistRepository.UserData user = overrideCheck
                    ? new WhitelistRepository.UserData(player.getUniqueId(), player.getName(), "", "", 0L, 0L)
                    : this.fetchUserData(player.getUniqueId());

            if (timestampRelevant(user.banned_until())) {
                Instant bannedDate = new Date(user.banned_until() * 1000L)
                        .toInstant()
                        .plus(1, ChronoUnit.HOURS);

                DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("dd.MM.yyyy").withZone(ZoneOffset.UTC);
                DateTimeFormatter timeFormat = DateTimeFormatter.ofPattern("HH:mm").withZone(ZoneOffset.UTC);

                throw new DisconnectInfo.Throwable(
                        "Du wurdest vom Server gebannt.",
                        String.format("Dein Bann läuft am %s um %s ab!", dateFormat.format(bannedDate), timeFormat.format(bannedDate)),
                        "Wende dich an einen Admin für weitere Informationen.",
                        player.getUniqueId()
                );
            }

            queryAppliance(Outlawed.class).updateForcedStatus(player, timestampRelevant(user.outlawed_until()));

            String purePlayerName = Floodgate.isBedrock(player)
                    ? Floodgate.getBedrockPlayer(player).getUsername()
                    : player.getName();

            if (!user.username().trim().equalsIgnoreCase(purePlayerName))
                throw new DisconnectInfo.Throwable(
                        "Nutzername geändert",
                        String.format("Der Name '%s' stimmt nicht mit '%s' überein.", user.username(), player.getName()),
                        "Bitte kontaktiere einen Admin, um Deine Anmeldedaten zu aktualisieren!",
                        player.getUniqueId()
                );

        } catch(DisconnectInfo.Throwable e) {
            throw e;
        } catch(Exception e) {
            Main.instance().getLogger().log(Level.SEVERE, e, e::getMessage);
            throw new DisconnectInfo.Throwable(
                "Interner Serverfehler",
                "Deine Anmeldedaten konnten nicht abgerufen/ überprüft werden.",
                "Versuche es später erneut oder kontaktiere einen Admin!",
                player.getUniqueId()
            );
        }
    }

    private boolean timestampRelevant(Long timestamp) {
        if(timestamp == null) return false;
        return timestamp > System.currentTimeMillis() / 1000L;
    }

    private WhitelistRepository.UserData fetchUserData(UUID uuid) throws DisconnectInfo.Throwable {
        ReqResp<WhitelistRepository.UserData> response = this.queryRepository(WhitelistRepository.class).getUserData(uuid);

        if(response.status() == HttpStatus.NOT_FOUND)
            throw new DisconnectInfo.Throwable(
                "Nicht angemeldet",
                "Du bist derzeit nicht als Teilnehmer des CraftAttack-Projektes registriert!",
                "Melde Dich bei einem Admin für eine nachträgliche Anmeldung.",
                uuid
            );

        return response.data();
    }

    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        record User(UUID user) {
        }
        apiBuilder.post("update", User.class, (user, request) -> {
            Main.instance().getLogger().info(String.format("API Triggered Profile update for %s", user.user));
            Player player = Bukkit.getPlayer(user.user);
            if(player != null) {
                try {
                    this.integrityCheck(player);
                } catch(DisconnectInfo.Throwable e) {
                    e.getDisconnectScreen().applyKick(player);
                }
            }
            return HttpServer.nothing;
        });
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(
            new PlayerJoinListener()
        );
    }
}
